//LCD + BUTTON
////////////////////////////////////
//MEGA     / UNO
//4,5,6,7  / 4,5,6,7  -> LCD Data transmission D4-D7
//8        / 8        -> Register Select
//9        / 9        -> Enable pin
//10       / 10       -> Backlight control
//A0       / A0       -> Button


//TRIAC
///////////////////////////////////
//MEGA     / UNO
//22       / 11       -> Bomba recirculacion
//23       / 12       -> Soplador
//24       / 13       -> Resistencia


//RELAY
//////////////////////////////////
//MEGA     / UNO
//25       / -        -> Husillo resistencia 1
//26       / 2        -> Husillo resistencia 2
//27       / 3        -> Husillo pellet
//28       / -        -> Libre


//SENSORS
//////////////////////////////////
//MEGA     / UNO
//30       / A2        -> Sensor Tª caldera DS18B20
//A1       / A1        -> Sensor llama

//31        -> Sensor Tª exterior DS18B20
//32        -> Sensor Husillo pellet
//33        -> Interruptor modo manual
//34        -> Sensor ultrasonido HC-SR04 cantidad pellet en tolva



////////////////////////////////////////////////////////////////////
// PANTALLA INICIAL EN REPOSO
////////////////////////////////////////////////////////////////////
// CALDERA XX:XX:XX
// APAGADA T: XX.XX
///////////////////////////////////////////////////////////////////



#include <LiquidCrystal.h>
#include <Wire.h>
#include "RTClib.h"
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <EEPROM.h>
#include <OneWire.h>
#include <DallasTemperature.h>


#define pulsadorLcd 0     //Entrada pulsador lcd
#define espera 1000       //Tiempo espera para actualizar reloj
#define retroIlumin 10    //Variable que determinar los segundos para apagar retroiluminacion y salir de menu
#define fuegoLimit 10     //Variable que indica el valor minimo para decir que hay llama
#define fuegoErrorLim 2   //Variable que indica el numero maximo de detecciones no fuego
#define testTime 30       //Variable que indica cada cuantos segundos realizamos los test, bajamos a 30 por los horarios

// SI USAMOS UN ARDUINO MEGA
#define triacBomb 22    //Salida triac para bomba recirculacion
#define triacSopla 23   //Salida triac para soplador
#define triacResis 24   //Salida triac resistencia

#define releHusiRes1 25 //Salida rele1 husillo resistencia
#define releHusiRes2 26 //Salida rele2 husillo resistencia
#define releHusiPel 27  //Salida rele husillo pellet
#define releLibre 28    //Salida rele reserva

#define pulsMan  29     //Pulsador marcha/paro de forma manual
#define sondaCald 30    //Sonda Temperatura Caldera 
#define sensLlama A1    //Sensor intensidad llama
#define sensUltras 32   //Sensor ultrasonico monitorizar carga tolva

#define pulManual 19    //Pulsador Manual - Conectado a pin con interrupcion INT3
#define sensHusPel 18   //Sensor vueltas husillo Pellet - Conectado a pin con interrupcion INT2

#define lightLcd 10     //Salida light retroiluminacion lcd


// SI USAMOS UN ARDUINO UNO
/*
#define triacBomb 11    //Salida triac para bomba recirculacion
#define triacSopla 12   //Salida triac para soplador
#define triacResis 13   //Salida triac resistencia

#define releHusiRes 2   //Salida rele husillo resistencia
#define releHusiPel 3   //Salida rele husillo pellet


#define sondaCald A2    //Sonda Temperatura Caldera 
#define sensLlama A1    //Sensor intensidad llama
#define sensUltras A3   //Sensor ultrasonico monitorizar carga tolva
*/



OneWire  cald(30);      //Pin 30. Es necesaria una resistencia de 4.7K
DallasTemperature caldSens(&cald);

//Inicializar libreria con los pines correctos
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

//RTC
RTC_DS1307 rtc;
DateTime now;



unsigned long tiempo;   //Variable usada para almacenar millis()
int estado = 0;         //Variable usada en funcion switch-case
int boton = -1;         //Variable usada para almacenar el boton pulsado
int hora, minuto, segundo, dia, mes, ano;

int horaIni1, minutoIni1, horaFin1, minutoFin1;    //Variables que almacenan las configuraciones del horario uno
int horaIni2, minutoIni2, horaFin2, minutoFin2;    //Variables que almacenan las configuraciones del horario dos
int horaIni3, minutoIni3, horaFin3, minutoFin3;    //Variables que almacenan las configuraciones del horario tres
int horaIni4, minutoIni4, horaFin4, minutoFin4;    //Variables que almacenan las configuraciones del horario cuatro
int h1, m1, h2, m2;
boolean activo = false;   //Variable usada como buffer
boolean activo1 = false;  //Varible que indica si esta activa esta alarma
boolean activo2 = false;  //Varible que indica si esta activa esta alarma
boolean activo3 = false;  //Varible que indica si esta activa esta alarma
boolean activo4 = false;  //Varible que indica si esta activa esta alarma

boolean salir = false;    //Variable que indica si salir del menu
boolean retro = false;    //Variable que indica el estado de la retroiluminacion

boolean modoTermo = false;  //Variable que indica que hay un termo, entonces no conviene recircular hasta enfriar
boolean manual = true;      //Variable que indica si el arranque es manual o con horarios

int tPrebarrido = 0;        //Variable que almacena los segundos de prebarrido
int tResistencia = 0;       //Variable que almacena los segundos de encendido a traves de resistencia
int tPelletIniS = 0;        //Variable que almacena los segundos para carga inicial de pellet
int tVentiResis = 0;        //Variable que almacena el cuando se enciende el ventilador antes de retirar la resistencia
int tPelletManS = 0;        //Variable que almacena los segundos de carga de mantenimiento
int tPelletIntS = 0;        //Variable que almacena cada cuantos se realiza la carga de mantenimiento
int tPelletIntL = 0;        //Variable que almacen a cuánto debe bajar la intensidad de llama para realizar una carga de mantenimiento
int tPausaMax = 0;          //Variable que almacena el tiempo máximo de pausa sin arrancar quemador
int tLimpFinal = 0;         //Variable que almacena el tiempo de limpieza final. Se mantiene el soplador activo para terminar con los restos
int tempBombaMax = 0;       //Variable que almacena el umbral para arranque de bomba
int tempBombaMin = 0;       //Variable que almacena el umbral para parada de bomba
int tempCalderaMin = 0;     //Variable que almacena umbral para parada bomba
int tempCalderaMax = 0;     //Variable que almacena umbral para quemador en modo espera
int tempCalderaError = 0;   //Variable que almacena umbral maximo funcionamiento caldera
int tempViviendaMax = 0;    //Variable que almacena umbral maximo temperatura vivienda
int tempViviendaMin = 0;    //Variable que almacena umbral minimo temperatura vivienda
int tempExterior = 0;       //Variable que almacena umbral temperatura exterior

float tempCaldera = 0;        //Variable que almacena la temperatura de la caldera actual
boolean encendida = false;    //Variable que indica si la caldera esta encendida o apagada. SI SE REINICIA EMPIEZA SIEMPRE APAGADO
boolean prebarrido = false;   //Variable que indica si se ha realizado el prebarrido
boolean encendiendo = false;  //Variable que indica si estamos en la funcion encendiendo
int contadorS = 0;            //Variable usada para contar segundos
int contadorP = 0;            //Variable usada para contar segundos de suministro pellet
int valorLlama = 0;           //Variable usada para almacenar valor leido de llama


boolean temperatura = false;  //Variable que indica si se ha alcanzado temperatura de espera quemador
boolean fuego = false;        //Variable que indica si hay llama


boolean estResis = false;       //Variable que indica si la resistencia esta activa
boolean estHusiResis = false;   //Variable que indica si el actuador de la resistencia esta activo
boolean estHusiPellet = false;  //Variable que indica si el husillo alimentador de pellet esta activo
boolean estSopl = false;        //Variable que indica si el soplador esta funcionando
boolean estBomba = false;       //Variable que indica si la bomba esta funcionando
boolean cargaPellet = false;    //Variable que indica si se esta cargando pellet

int inactividad = 0;          //Variable que se usa como contador de inactividad
int testSensor = 0;           //Variable usada como contador para entrar en test
int timeHour = 0;             //Variable usada para almacenar hora y comprobar que el reloj funciona
int timeMinute = 0;           //Variable usada para almacenar hora y comprobar que el reloj funciona
int contadorFuego = 0;        //Variable que almacena las veces que no se detecta fuego. Cuando llegue al limite error.

boolean error = false;        //Variable que indica que hay algun error
boolean tempError = false;    //Variable que indica si se ha alcanzado la temperatura máxima de seguridad. Parada total de la caldera
boolean fuegoError = false;   //Variable que indica que no se ha podido iniciar fuego
boolean pelletError = false;  //Variable que indica que nos hemos quedado sin pellet
boolean sondaError = false;   //Variable que indica que no hay comunicación con la sonda de temperatura
boolean rtcError = false;     //Variable que indica que el rtc esta parado

boolean ctrLlama = false;     //Variable que indica si se alimenta por tiempo o por intensidad llama
boolean segLlama = false;     //Variable que indica si se alimenta por intensidad de llama o por tiempo
boolean tempResidual = false; //Variable que indica que se ha apagado el quemador, pero debe seguir activa la bomba para enfriar
boolean apagando = false;     //Variable que indica que debemos realizar la limpieza antes de apagar

String espHora;               //Variable que almanena la cadena HORA a enviar a la web
String espEstado = "Apagada"; //Variable que almanena la cadena ESTADO a enviar a la web: PREBARRIDO, ENCENDER, ENCENDIDA, ESPERA, LIMPIANDO, APAGADA, ERROR
String espValor = "--";       //Variable que almanena la cadena VALOR a enviar a la web: TIEMPOS DE LOS ESTADOS, TIPO DE ERROR
String espTemp;               //Variable que almanena la cadena TEMPERATURA a enviar a la web
String espLux;                //Variable que almanena la cadena LUX a enviar a la web
String espBomba = "Off";      //Variable que almanena la cadena BOMBA a enviar a la web



void setup() 
{
  //Inicializamos la comunicación serie
  Serial.begin(9600);     //Usb debug
  Serial2.begin(9600);    //Puerto comandos - Conexion con ESP8266
  
  //Inicializados lcd columnas y filas
  lcd.begin(16, 2);
  
  //Sacamos versión por display
  lcd.print("CALDERA V2.0");
  delay(500);
  
  //Lcd backlight
  pinMode(lightLcd, OUTPUT);
  digitalWrite(lightLcd, HIGH);
  retro = true;

  //Configuramos las salidas
  pinMode(triacBomb, OUTPUT);
  pinMode(triacSopla, OUTPUT);
  pinMode(triacResis, OUTPUT);

  pinMode(releHusiRes1, OUTPUT);
  pinMode(releHusiRes2, OUTPUT);
  pinMode(releHusiPel, OUTPUT);
//  pinMode(releLibre, OUTPUT);

  //Configuramos entradas
  pinMode(pulsMan, INPUT_PULLUP);

  //Inicializamos salidas todo apagado
  digitalWrite(triacBomb, LOW);
  digitalWrite(triacSopla, LOW);
  digitalWrite(triacResis, LOW);
  digitalWrite(releHusiRes1, HIGH);
  digitalWrite(releHusiRes2, HIGH);
  digitalWrite(releHusiPel, HIGH);


 
  //Cargamos la configuracion almacenada en la EEPROM
  readConfig();

  //Inicializamos la sonda de temperatura de la caldera
  caldSens.begin();
  
  //Inicializamos el rtc
  rtc.begin();
  //rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  delay(500);
  if (! rtc.isrunning())    //Si esta parado el rtc
  {
  rtcError = true;
  lcd.print("RTC NO RUNNING");
  }

  //Cargamos en la variable el valor actual de millis()
  tiempo = millis();
}





void loop ()
{

  switch (estado)
  {

    case 0:     //CASE PRINCIPAL DE CONTROL
    {
      //////////////////////////////////////////////////////////////////// ENTRAMOS SI HA PASADO UN SEGUNDO
      if (espera < (millis() - tiempo))   
      { 
      Serial.println("1s actualizando");
      tiempo = millis();
      updateTime();                     //Cada segundo actualizamos hora en display
      valorLlama = leerIntensLlama();   //Cada segundo actualizamos intensidad llama
      tempCaldera = leerTempCald();     //Cada segundo leemos la temperatura de la caldera
      inactividad++;                    //Cada segundo incrementamos el contador de inactividad
      testSensor++;                     //Cada segundo incrementamos el contador de test Sensores
      
        if (encendida)                  //Si la caldera esta encendida
        {
        contadorS++;                    //Cada segundo incrementamos contadorS que es usando en los tiempos de maniobras
        contadorP++;                    //Cada segundo incrementamos contadorP que es usando para el tiempo de carga pellet
        }

      updateDisplay();                  //Cada segundo actualizamos el display

      verErrores ();                    //Comprobamos si hay errores
      }  
       

      //////////////////////////////////////////////////////////////////// ENTRAMOS SI HA PASADO EL TIEMPO DE ESPERA TEST      
      if (testSensor > testTime)       
      {
      Serial.println("Realizando Test Sensores");
      testSensor = 0;
      sondaTest();          //Comprobamos que la sonda de temperatura responda correctamente
      relojTest();          //Comprobamos que el reloj funcione correctamente
      if (!manual)          //Si tenemos activo el modo automatico, comprobamos los horarios
      horarioComp();
      } 

      //////////////////////////////////////////////////////////////////// ENTRAMOS SI SE HA PULSADO EL BOTON MARCHA/PARO
      if (!digitalRead(pulsMan) && !error)     //Si se ha pulsado boton marcha/paro y no hay error nos vamos al menu de marha paro por boton
      {
      marchaParo();
      }

      //////////////////////////////////////////////////////////////////// ENTRAMOS SI SE HA PULSADO EN EL TECLADO
      if ((boton = read_LCD_buttons()) != -1)     // Comprobamos si hay pulsacion
      {                                           
        if (encendida)    //Si la caldera esta encendida, no entramos en el menu
          {
          lcd.clear();
          lcd.print ("CALDERA ON!");
          delay(1000);
          tiempo = millis();            
          }
        else              //Si no esta encendida
          {
          delay(300);
          menuConf();
          delay(100);
          tiempo = millis();
          } 
          estado = 0;
       }

      //////////////////////////////////////////////////////////////////// ENTRAMOS PARA APAGAR LA RETROILUMINACION
      if (inactividad>retroIlumin && !encendida && !tempResidual && retro)    //Si ha pasado el tiempo de inactividad, y no está encendida la caldera, apagamos la retroiluminacion
      {
      digitalWrite(lightLcd, LOW);     //Apagamos retroiluminacion lcd 
      retro = false;
      }

      //////////////////////////////////////////////////////////////////// ENTRAMOS SI HAY ACTIVIDAD EN EL PUERTO SERIE - ACTUALIZAR WEB ESP8266
      if((Serial2.available() > 0))   //Si hemos recibido algo por puerto serie, vamos a trabajar con ello
      {
        Serial.println("SERIE REC");
        cmdSerial();
      }

      //////////////////////////////////////////////////////////////////// ENTRAMOS PARA IR AL CASE QUE CORRESPONDE
      if (!encendida)   //Si no esta encendida
      {
        if (!tempResidual)    //Si no hay temperatura residual, nos mantenemos en el case 0
        estado = 0;
        else                  //Si hay temperatura residual nos vamos al case 6
        estado = 6;
      }

      else              //Si esta encendida
      {
        if (!prebarrido && !apagando)         //No ha concluido el prebarrido, saltamos para concluirlo
          {
          //Serial.println ("PREBARRIDO");
          digitalWrite(lightLcd, HIGH);     //Encendemos retroiluminacion lcd 
          retro = true;
          estado = 1;
          }
        else if (prebarrido && !encendiendo && !apagando)   //Ha concluido el prebarrido pero no el encendiendo, saltamos a encender
          {
          //Serial.println ("ENCENDIENDO");
          estado = 2;
          }
        else if (prebarrido && encendiendo && !apagando)    //Ha concluido el prebarrido y el encendido, entonces estamos en mantenimiento
          {
          //Serial.println ("MANTENIMIENTO");
          estado = 3;
          }
        else if (apagando)                     //Si se habilita apagando, procedemos a realizar el apagado
          {
          //Serial.println ("LIMPIEZA");
          estado = 4;
          }
      }

    }break;



    case 1:     //CASE PREBARRIDO
    {   
      if ((tPrebarrido < contadorS) || (fuego && segLlama))     //Esperamos hasta que pase el tiempo de prebarrido o se haya detectado fuego
      {
        lcd.setCursor(0,1);             //Se ha terminado por tiempo
        Serial.println("FIN PREBAR  ");
        delay(500);
        prebarrido = true;              //Indicamos que el prebarrido ha finalizado     
        if (fuego && segLlama)          //Si ha terminado porque se detecta fuego
        {
          lcd.setCursor(0,1);
          lcd.print("HAY FUEGO   ");
          Serial.println("HAY LLAMA EN PREBARRIDO!! ");
          delay(500);
          encendiendo = true;             //Como hay fuego, nos saltamos el paso de encender
        }
        marchaParoSoplador(false);      //Apagamos soplador
      }
      else        //Si no hemos terminado con el prebarrido
      {
        if(!estSopl)      //Si esta apagado el soplador, es la primera vez que entramos entonces encendemos
        {
          Serial.println("PREBARRIDO  ");
          marchaParoSoplador(true);         //Encendemos soplador
          contadorS = 0;
        } 
      }
    estado = 0; 
    }break;


    case 2:   //ENCENDIENDO - HEMOS REALIZADO EL PREBARRIDO Y NO HEMOS DETECTADO FUEGO
    {
      if ((tResistencia < contadorS) || (fuego && segLlama))    //No apagamos resistencia hasta que haya pasado su tiempo o detectemos llama
      {
      Serial.println("FIN ENCENDIDO");
      marchaParoResist(false);      //Apagamos resistencia
      marchaParoHusiResist(false);  //Sacamos resistencia
      marchaParoSoplador(false);    //Paramos soplador
      
        if ((tResistencia < contadorS))   //Si pasamos al estado encendido por tiempo, comprobamos si hay fuego si esta habilitada la funcion
        {
          if(!fuego && segLlama)  //Si en este punto no se detecta fuego y esta habilitada la funcion, apagamos y mostramos error.
          {
          fuegoError = true; 
          apagTodo();
          }
        }
        encendiendo = true;     //Indicamos que el encendido ha finalizado
      }
      else          //Si aun no ha terminado el encendido
      {
        if (!estResis)      //Si esta apagada la resistencia, acabamos de entrar y debemos poner todo a funcionar
        {
        Serial.println("ENCENDER    ");
        marchaParoHusiResist(true); //Llevamos la resistencia hasta su posicion
        marchaParoResist(true);     //Encendemos resistencia
        marchaCargaPellet(true);    //Iniciamos carga pellet
        contadorS = 0;
        }

        
        if (cargaPellet)    //Si hemos habilitado la carga de pellet
        {
          if ((tPelletIniS < contadorP) || (fuego && segLlama))     //Si ha pasado el tiempo de carga, o ya se ha detectado fuego
          {
          marchaCargaPellet(false);    //Paramos carga pellet
          Serial.println("FIN CARGA PELLET");
          }
        }
     

        if (((tResistencia - contadorS) < tVentiResis) && !estSopl)   //Arranca ventilador x segundos antes de retirar la resistencia
        {
          marchaParoSoplador(true);     //Encendemos soplador
        }
      }
      
      estado = 0;
      }break;



     case 3:   //MANTENIMIENTO - MANTENEMOS EL FUEGO ACTIVO
     {
        if(!estSopl && !temperatura)      //Si el soplador esta apagado, y no es por pausa de temperatura, es que acabamos de entrar
        {
        marchaParoSoplador(true);     //Encendemos soplador
        marchaCargaPellet(false);     //No Iniciamos carga pellet - funciona mejor dejando arrancar bien el fuego. Al cargar se achica.
        contadorS = 0;
        }
         

        if (((tPelletIntS < contadorS) || cargaPellet)&& !temperatura )    //Si ha pasado el tiempo de espera para carga, o esta habilitada carga inmediata, se carga 
        {                                                                  //Si se para por temperatura, no carga
          if (ctrLlama && !cargaPellet)           // Si esta habilitado el control de llama y no se fuerza carga, comprobamos intensidad llama para cargar
          {                               
            if (valorLlama < tPelletIntL)   //Si la intensidad de llama ha bajado por debajo del umbral
            {
            marchaCargaPellet(true);        //Iniciamos carga pellet
            }
          }
          else            // Si se esta cargando, comprobamos para parar
          {       
            if ((tPelletManS < contadorP))     //Si ha pasado el tiempo de carga 
            {
            contadorS = 0;                //Ponemos a cero la espera hasta la proxima carga
            marchaCargaPellet(false);     //Paramos carga pellet
            Serial.println("FIN CARGA PELLET");
            }
          }
        }




     if ((tempCalderaMax < tempCaldera) && estSopl && !estHusiPellet && !apagando)    //Si se ha alcanzado la temperatura maxima, paramos el fuego
     {                                                                                //Si estamos cargando, esperamos a que termine
      Serial.println("TEMPERATURA MAXIMA ALCANZADA - PARANDO VENTILADOR");
      temperatura = true;
      marchaParoSoplador(false);    //Paramos soplador
      contadorS = 0;
     }



     if ((temperatura && (tempCaldera < tempCalderaMin)) || (tPausaMax < contadorS))   //Si ha bajado la temperatura hasta el mínimo o ha pasado el tiempo maximo de quemador apagado
     {
      Serial.println("ARRANCAR QUEMADOR PARA MANTENIMIENTO");
      marchaParoSoplador(true);    //Arrancamos soplador
      temperatura = false;
      marchaCargaPellet(true);     //Iniciamos carga pellet
      contadorS = 0;
     }
     

     if (tempCalderaError < tempCaldera)     //Si sobrepasa la temperatura máxima, apaga todo, reinicia y se queda indicando error.
     {
      Serial.println("PELIGRO - TEMPERATURA MAXIMA ALCANZADA");
      apagTodo();
      tempError = true;
     }


        if (segLlama && !temperatura && (contadorS == tPelletIntS))       //Cada vez que toque cargar, si esta habilitada la seguridad y no hemos alcanzado la temperatura de pausa, 
        {                                                                 //comprobamos si hay fuego
          delay(500);
          if(!fuego)  //Si no se detecta llama, seguramente haya problemas con la alimentacion de pellet
            {                                         //Comprobamos llama cada vez que toque recargar
              if (contadorFuego < fuegoErrorLim)      //Si sobrepasamos las veces máximas de no deteccion de llama
              {
              lcd.setCursor(0,1);
              lcd.print("ERROR FUEGO ");
              delay(500);
              contadorFuego ++;
              }
              else
              {
              pelletError = true; 
              apagTodo();
              }
            }
          else
            {
            contadorFuego = 0;
            lcd.setCursor(0,1);
            lcd.print("HAY FUEGO   ");
            delay(500);
            }
        }

     estado = 6;      //Vamos a comprobar la bomba
    }break;



    case 4:   //LIMPIEZA - NOS ASEGURAMOS DE QUEMAR TODO EL PELLET DEL QUEMADOR ANTES DE APAGAR
    {
      if (apagando)      //Si se ha indicado apagar caldera, mantenemos el soplador el tiempo definido antes de apagar todo
     {
      if (!(estSopl && !estResis && !estHusiResis && !estHusiPellet)) 
        {
        marchaParoSoplador(true);     //Arrancamos soplador
        marchaCargaPellet(false);     //Forzamos parada carga pellet
        marchaParoHusiResist(false);  //Sacamos resistencia
        marchaParoResist(false);      //Apagamos resistencia
        }

      
      if (tLimpFinal < contadorS)   //Termina el tiempo de limpieza
      {
        marchaParoSoplador(false);     //Paramos soplador
        encendida = false;
        apagando = false;  
      }
     }
     estado = 5;
    }break;
    

    case 5:   //APAGAR - NOS ASEGURAMOS DE APAGAR TODO
    {
     if (!encendida)        //Si se ha apagado la caldera, apagamos el soplador
     {
      Serial.println("APAGANDO");
      apagTodo();
     }
     estado = 0;
    }break;

    
    case 6:   //CONTROL - BOMBA
    {
     // Serial.println ("Comprobando bomba");
     if ((tempBombaMax < tempCaldera) && !estBomba)     //Si se ha superado la temperatura maxima, enciende la bomba
     {
      espBomba = "On";
      Serial.println("TEMPERATURA ALCANZADA - ENCENDIENDO BOMBA");
      marchaParoBomba(true);
        if (!modoTermo)       //Si no esta modo termo activo, habilita temperatura residual
        tempResidual = true;
     }

     if ((tempCaldera < tempBombaMin) && estBomba)    //Si la temperatura ha bajado por debajo del umbral minimo, paramos la bomba
     {
      espBomba = "Off";
      Serial.println("TEMPERATURA BAJA - PARAR BOMBA");
      marchaParoBomba(false);
      tempResidual = false;
     }

    estado = 0;
    }break; 
    
  }
}



  
    
        
////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE INICIAR CARGA PELLET
////////////////////////////////////////////////////////

void marchaCargaPellet(boolean marParo)
{
 if (marParo)
 {
  cargaPellet = true;                  //Bandera que indica que podemos cargar
  contadorP = 0;                       //Ponemos a cero contador carga pellet
  digitalWrite(releHusiPel, LOW);      //Activamos el husillo 
  estHusiPellet = true;               //Variable que nos indica que estamos cargando pellet
  //lcd.setCursor(0,1);
  //lcd.print("PELLET ON   ");
  Serial.println("PELLET ON     ");
 }
 else
 {
  cargaPellet = false;                  //Bandera que indica que no debemos cargar
  contadorP = 0;                        //Ponemos a cero contador carga pellet
  digitalWrite(releHusiPel, HIGH);      //Activamos el husillo 
  estHusiPellet = false;               //Variable que nos indica que hemos terminado de cargar pellet
  //lcd.setCursor(0,1);
  //lcd.print("PELLET OFF  ");
  Serial.println("PELLET OFF  ");
 }
 delay (500);
}




////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE ARRANCAR O PARAR BOMBA
////////////////////////////////////////////////////////

void marchaParoBomba(boolean marParo)
{
  if (marParo)
  {
    digitalWrite(triacBomb, HIGH);      //Encendemos bomba
    estBomba = true;                    //Variable que nos indica que la bomba esta encendida
    lcd.setCursor(0,1);
    lcd.print("BOMBA ON    ");
    Serial.println("BOMBA ON    ");
  }
  else
  {
    digitalWrite(triacBomb, LOW);       //Apagamos bomba
    estBomba = false;                   //Variable que nos indica que la bomba esta apagada
    lcd.setCursor(0,1);
    lcd.print("BOMBA OFF   ");
    Serial.println("BOMBA OFF   ");
  }
  delay (500);
}

      
////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE MOVER HUSILLO RESISTENCIA
////////////////////////////////////////////////////////

void marchaParoHusiResist(boolean marParo)
{
  if (marParo)
  {
    digitalWrite(releHusiRes1, LOW);    //Llevamos la resistencia hasta su posicion
    delay(500);
    digitalWrite(releHusiRes2, HIGH);   //Llevamos la resistencia hasta su posicion
    estHusiResis = true;                //Variable que nos indica que la resistencia esta dentro
    //lcd.setCursor(0,1);
    Serial.println("RESIST IN   ");
  }       
  else 
  {
    digitalWrite(releHusiRes1, HIGH);   //Retiramos resistencia
    delay(500);
    digitalWrite(releHusiRes2, LOW);    //Retiramos resistencia
    estHusiResis = false;
    //lcd.setCursor(0,1);
    Serial.println("RESIST OUT  ");
  }
  delay (500);
}



////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE ARRANCAR O PARAR RESISTENCIA
////////////////////////////////////////////////////////

void marchaParoResist(boolean marParo)
{
  if (marParo)
  {
    digitalWrite(triacResis, HIGH);  //Encendemos resistencia
    estResis = true;
    //lcd.setCursor(0,1);
    //lcd.println("RESIST ON   ");
    Serial.println("RESIST ON   ");
  }
  else
  {
    digitalWrite(triacResis, LOW);  //Apagamos resistencia
    estResis = false;
    //lcd.setCursor(0,1);
    //lcd.println("RESIST OFF  ");
    Serial.println("RESIST OFF  ");
  }
  delay (500);
}


////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE ARRANCAR O PARAR SOPLADOR
////////////////////////////////////////////////////////

void marchaParoSoplador(boolean marParo)
{
  if (marParo)
  {
    digitalWrite(triacSopla, HIGH);  //Encendemos ventilador
    estSopl = true;
    //lcd.setCursor(0,1);
    //lcd.print("SOPLADOR ON ");
    Serial.println("SOPLADOR ON ");
  }
  else
  {
    digitalWrite(triacSopla, LOW);  //Apagamos ventilador
    estSopl = false;
    //lcd.setCursor(0,1);
    //lcd.print("SOPLADOR OFF");
    Serial.println("SOPLADOR OFF");
  }
  delay (500);
}



////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE ARRANCAR O PARAR POR BOTON
////////////////////////////////////////////////////////

void marchaParo()
{
  while(!digitalRead(pulsMan))
    {
    delay(100);
    }
    
  if(!encendida)        //Si no esta encendida, la encendemos
    encendida = true;
  else                  //Si esta encendida comprobamos en que punto
    {
    if (encendida && !apagando && prebarrido)  //Si esta encendida, no esta apagando y no estamos en prebarrido
      {
      apagando = true;
      contadorS = 0;
      Serial.println ("No estamos en limpieza. Apagando = true");
      }
    else    //Si esta encendida y esta apagando o no habia pasado de prebarrido
      {
      Serial.println ("Estamos en limpieza. Encendida = false");
      encendida = false;
      apagTodo();
      }
    }
}



////////////////////////////////////////////////////////
//    FUNCION ENCARGADA DE COMPROBAR ERRORES 
////////////////////////////////////////////////////////

void verErrores ()
    {
      if (fuegoError || tempError || pelletError || sondaError || rtcError)   //Si hay algun error, lo muestra y dejan de funcionar algunas funciones hasta el reseteo
        {
        espEstado = "Error";
        error = true;
        digitalWrite(lightLcd, HIGH);     //Encendemos retroiluminacion lcd 
        retro = true;
        lcd.clear();
        lcd.print("ERROR"); 
        lcd.setCursor(0,1);
        if(fuegoError)
        {
        espValor = "ERROR INIC FUEGO";
        }
        if(tempError)
        {
        espValor = "ERROR TEMPERATUR";
        }
        if(pelletError)
        {
        espValor = "ERROR PELLET    ";
        }
        if(sondaError)
        {
        espValor = "ERROR SONDA TEMP";
        }
        if(rtcError)
        {
        espValor = "ERROR RELOJ     ";
        }
        lcd.print(espValor);
        delay(500);
        apagTodo();
        inactividad = 0;  //Para no apagar display por inactividad
        }
    }


////////////////////////////////////////////////////
//    PRESENTAR DIGITOS DISPLAY
////////////////////////////////////////////////////

void escribirDigitos(String titulo, int ndig, int pos, int linea, int dig)   //titulo = indica que vamos a presentar, ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
{
  if (titulo != -1)
  {
  lcd.setCursor(0,1);
  lcd.print(titulo);
  }
  if(dig>0)
  {
  lcd.setCursor(pos,linea);
  while(ndig > 0)
  {
    int valor = 1;
    for(int i = 0; i < ndig-1; i++)
    {
      valor = valor * 10;
    }
  if (dig < valor)
  lcd.print("0");
  ndig--;
  }
  lcd.print(dig);
  }
  else
  {
    lcd.setCursor(pos,linea);
     for(int i = 0; i < ndig; i++)
    {
      lcd.print("-");
    }
  }
}





////////////////////////////////////////////////////
//    COMUNICACION SERIE CONTROL Y MONITORIZACION
////////////////////////////////////////////////////

//CALDEST -> DEVOLVER ESTADO DE LA CALDERA: EST,HORA,ESTADO,VALOR,TEMPERATURA,LUX,BOMBA
//CALDON -> ENCENDER
//CALDOFF -> APAGAR
//CALDRESET -> RESET

void cmdSerial()       //Si se detecta entrada de datos por el puerto, entra en esta funcion
{
  String comandoRecib = "";
  char data=' ';
  int maxEsper = 0;
  while(((data = Serial2.read()) != '\n') && (maxEsper < 500) )   //Recogemos datos mientras no haya fin de linea o se haya sobrepasado el tiempo maximo de espera
  {
   comandoRecib += data;
   delay(10);
   maxEsper++;
  }
  if(comandoRecib.indexOf("CALD") >-1 )    //Si el comando empieza con la cadena "CALD" es correcto
  {
    if(comandoRecib.substring(4,6) == "ON")        //Comando encender caldera
    {
      encendida = true;
    }
  else if (comandoRecib.substring(4,7) == "OFF")  //Comando apagar caldera
  {
      if (encendida && !apagando && prebarrido)  //Si esta encendida, no esta apagando y no estamos en prebarrido
        {
        apagando = true;
        contadorS = 0;
        Serial.println ("No estamos en limpieza. Apagando = true");
        }
      else    //Si esta encendida y esta apagando o no habia pasado de prebarrido
        {
        Serial.println ("Estamos en limpieza. Encendida = false");
        encendida = false;
        apagTodo();
        }
  }
  else if (comandoRecib.substring(4,9) == "RESET")  //Comando resetear caldera
  {
  apagTodo();
  resetSistema();
  }
  else if (comandoRecib.substring(4,7) == "EST")  //Comando preguntar estado
  {
  char fecha[5];
  int hora = now.hour();
  int minuto = now.minute();
  int segundo = now.second();
  sprintf( fecha, "%.2d:%.2d:%.2d", hora, minuto, segundo);
  espLux = String (valorLlama);
  espTemp = String (tempCaldera);
  
      Serial2.print("CALDEST,");
      Serial2.print(fecha);
      Serial2.print(",");
      Serial2.print(espEstado);
      Serial2.print(",");
      Serial2.print(espValor);
      Serial2.print(",");
      Serial2.print(espTemp);
      Serial2.print(",");
      Serial2.print(espLux);
      Serial2.print(",");
      Serial2.println(espBomba);
    }
  }
}




////////////////////////////////////////////////////
//    APAGAR TODO
////////////////////////////////////////////////////

void apagTodo()
{
  Serial.println("APAGANDO TODO");
  if (!tempResidual && estBomba) //Si no hay temperatura residual y la bomba esta encendida, la paramos
    {
    marchaParoBomba(false);
    }
  marchaParoSoplador(false);    //Paramos soplador
  marchaParoResist(false);      //Paramos resistencia
  if (estHusiResis)             //Si esta metida la resistencia, la sacamos
    {
    marchaParoHusiResist(false);
    }
  else                          //Si no esta metida, apagamos el husillos
    {
    digitalWrite(releHusiRes1, HIGH);   //Apagamos husillo resistencia
    delay(500);
    digitalWrite(releHusiRes2, HIGH);   
    Serial.println("APAGANDO HUSILLO RESISTENCIA");
    }
  marchaCargaPellet(false);     //Nos aseguramos de que no esta activo el husillo de pellet
  delay(500);


  //Reiniciamos variables estado secuencia 
  encendida = false;    //Variable que indica si la caldera esta encendida o apagada. SI SE REINICIA EMPIEZA SIEMPRE APAGADO
  prebarrido = false;   //Variable que indica si se ha realizado el prebarrido
  temperatura = false;  //Variable que indica si se ha alcanzado temperatura de espera quemador
  fuego = false;        //Variable que indica si hay llama
  apagando = false;     //Variable que indica que estamos en tiempo de limpieza antes de apagado
  encendiendo = false;  //Variable que indica que no se ha terminado el encendido
  contadorS = 0;
  contadorP = 0;
  contadorFuego = 0;
  espEstado = "Apagada";
}


////////////////////////////////////////////////////
//    LEER INTENSIDAD LLAMA
////////////////////////////////////////////////////

int leerIntensLlama()
{
  int valor = 0;
  valor = analogRead(sensLlama); 
  valor = map(valor, 0, 1024, 0, 100);

  if(valor < fuegoLimit)
  fuego = false;
  else
  fuego = true;
  return valor;
}


////////////////////////////////////////////////////
//    LEER TEMPERATURA CALDERA
////////////////////////////////////////////////////

float leerTempCald()
{
  caldSens.requestTemperatures(); // Send the command to get temperatures
  delay(100);
  return caldSens.getTempCByIndex(0); 
}


////////////////////////////////////////////////////
//    COMPROBAR ESTADO SONDA Tª
////////////////////////////////////////////////////

void sondaTest()
{
  caldSens.requestTemperatures(); // Send the command to get temperatures
  delay(200);
  if (caldSens.getTempCByIndex(0) < -15)  // Si devuelve un valor inferior a este, entendemos error. El valor que devuelve si no hay comunicación es -127
    {
      delay (200);
      caldSens.requestTemperatures(); // Send the command to get temperatures
      delay(200);
      if (caldSens.getTempCByIndex(0) < -15)
      sondaError = true;
    }
   
}



      
////////////////////////////////////////////////////
//    COMPROBAR ESTADO RELOJ
////////////////////////////////////////////////////

void relojTest()
{
    if (! rtc.isrunning())    //Si esta parado el rtc
    rtcError = true;
}




///////////////////////////////////////////////////////////////////
//    COMPROBAR SI ES HORA ACTIVACION/DESACTIVACION POR HORARIO
///////////////////////////////////////////////////////////////////

void horarioComp()
{
    boolean preparApagado = false;
    if (!encendida && !error)      //Si no está encendida y no hay error
    {
      if(horaIni1 == now.hour() && minutoIni1 == now.minute() && activo1)   //Si coincide la hora y el minuto de inicio con la hora y minuto actual, encendemos.
      encendida = true;
      else if (horaIni2 == now.hour() && minutoIni2 == now.minute() && activo2)
      encendida = true;
      else if (horaIni3 == now.hour() && minutoIni3 == now.minute() && activo3)
      encendida = true;
      else if (horaIni4 == now.hour() && minutoIni4 == now.minute() && activo4)
      encendida = true;
    }
    else                  //Si está encendida
    {
      if(horaFin1 == now.hour() && minutoFin1 == now.minute() && activo1)   //Si coincide la hora y el minuto de apagado con la hora y minuto actual, apagamos.
      preparApagado = true;
      else if (horaFin2 == now.hour() && minutoFin2 == now.minute() && activo2)
      preparApagado = true;
      else if (horaFin3 == now.hour() && minutoFin3 == now.minute() && activo3)
      preparApagado = true;
      else if (horaFin4 == now.hour() && minutoFin4 == now.minute() && activo4)
      preparApagado = true;  
    }

    if (preparApagado)    //Si esta activa esta opcion, preparamos el apagado
    {
      if (encendida && !apagando && prebarrido)  //Si esta encendida, no esta apagando y no estamos en prebarrido, entonces hacemos limpieza
        {
        apagando = true;
        contadorS = 0;
        Serial.println ("No estamos en limpieza. Apagando = true");
        }
    }
}



////////////////////////////////////////////////////
//    ACTUALIZAR DISPLAY
////////////////////////////////////////////////////

void updateDisplay()
{
  if (encendida)      //Si esta encendida
  {
    if (!prebarrido && !apagando)     //Si estamos en prebarrido y no estamos apagando
        {
        escribirDigitos("PREBA:      ",2, 6, 1, (tPrebarrido - contadorS));   //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
        espEstado = "Prebarrido";                           //Indicamos el estado para la web
        espValor = String (tPrebarrido - contadorS);        //Actualizamos los valores a mostrar
        }
    else if (prebarrido && !encendiendo && !apagando)   //Si estamos encendiendo y no estamos apagando
        {
        escribirDigitos("ENCEN:      ", 2, 6, 1, (tResistencia - contadorS));   //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
        espEstado = "Encendiendo";
        espValor = String (tResistencia - contadorS);
        }
    else if (prebarrido && encendiendo && !apagando)    //Si estamos en mantenimientos y no estamos apagando
        {
          if(!temperatura)
          {
            if(cargaPellet)     //Muestra el tiempo de carga
              escribirDigitos("CARGA:      ", 2, 6, 1, (tPelletManS - contadorP));   //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
            else
              escribirDigitos("ESPER:      ",2, 6, 1, tPelletIntS - contadorS);     //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
          espEstado = "Encendida";
          espValor = "--";
          }

          else   //Muestra el tiempo de pausa por temperatura
            {
            espEstado = "Pausa";
            espValor = String (tPausaMax - contadorS);
            escribirDigitos("PAUSA:      ", 2, 6, 1, tPausaMax - contadorS);   //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor
            }
        }
    else if (apagando)                     //Si estamos en limpieza y no estamos apagando
        {
        escribirDigitos("LIMPI:      ", 2, 6, 1, tLimpFinal - contadorS);   //ndig = digitos a cubrir, si no hay ponemos ceros, posicion, linea, valor  
        espEstado = "Limpiando";
        espValor = String(tLimpFinal - contadorS);
        }

   lcd.setCursor(0,0);             //Actualizamos temperatura y lux
   lcd.print("T:      ");
   lcd.setCursor(2,0);
   lcd.print(tempCaldera);
   lcd.setCursor(12,1);
   lcd.print("L:  ");
   lcd.setCursor(14,1);
   lcd.print(valorLlama); 
   inactividad = 0;                //Ponemos a cero el contador de inactividad para que no se apague la retroiluminacin
  }
  else              //Si esta apagada
  {
    lcd.setCursor(0,0);
    lcd.print("CALDERA");
    if (tempResidual)   //Si hay temperatura, la bomba sigue recirculando
    {
    lcd.setCursor(0,1);
    lcd.print("ENFRIAR T:      ");
    }
    else
    {
    lcd.setCursor(0, 1);
    lcd.print("APAGADA T:      ");
    }
   lcd.setCursor(11, 1); 
   lcd.print(tempCaldera); 
  }
}


////////////////////////////////////////////////////
//    ACTUALIZAR HORA
////////////////////////////////////////////////////

void updateTime()
{
  if(!(fuegoError || tempError || pelletError || sondaError))
  now = rtc.now();
  lcd.setCursor(8, 0);
  if(now.hour()<10)
  lcd.print("0");
  lcd.print(now.hour(),DEC);
  lcd.print(":");
  if(now.minute()<10)
  lcd.print("0");
  lcd.print(now.minute(),DEC);
  lcd.print(":");
  if(now.second()<10)
  lcd.print("0");
  lcd.print(now.second(),DEC);
  tiempo = millis();
  delay(50);
}



////////////////////////////////////////////////////
//    LEER TECLADO
////////////////////////////////////////////////////

int read_LCD_buttons()
{          
    int buttonRead = 1000;  
    buttonRead = analogRead(pulsadorLcd);       // Leemos el valor del pulsador
    delay(100);
    
    if (buttonRead < 1000)  //Si hubo pulsacion
      {
      if(!retro)       //Si la retroiluminacion esta apagada, la encendemos
        {   
        digitalWrite(lightLcd, HIGH);
        retro = true;
        inactividad = 0;  //Ponemos a cero el contador de inactividad para apagado de la retroiluminacion 
        buttonRead = 1000;
        delay(500);
        } 
      }
    
    if ((buttonRead > 0) && (buttonRead < 50) )   //RIGHT
    return 0; 
    else if ((buttonRead > 50) && (buttonRead < 150) )    //UP
    return 3; 
    else if ((buttonRead > 200) && (buttonRead < 300) )   //DOWN
    return 4; 
    else if ((buttonRead > 350) && (buttonRead < 450) )   //LEFT
    return 1; 
    else if ((buttonRead > 600) && (buttonRead < 700) )   //SELECT
    return 5; 
    else
    return -1;

    delay(100);           
}




////////////////////////////////////////////////////
//    REINICIAR CALDERA
////////////////////////////////////////////////////

void resetSistema()
{
  Serial.println("RESET");
  wdt_enable(WDTO_15MS);
  while(1)
  {
  }
}





